@@ -1,10 +1,10 @@
 
-# R CMD check NOTE handling
-if(getRversion() >= "2.15.1")  utils::globalVariables(c("."))
-
 ## partition
 #' @title Create balanced partitions.
-#' @description Splits data into partitions.
+#' @description
+#'  \Sexpr[results=rd, stage=render]{lifecycle::badge("stable")}
+#'
+#'  Splits data into partitions.
 #'  Balances a given categorical variable and/or numerical variable between partitions and keeps (if possible)
 #'  all data points with a shared ID (e.g. participant_id) in the same partition.
 #' @details
@@ -79,7 +79,7 @@ if(getRversion() >= "2.15.1")  utils::globalVariables(c("."))
 #' @author Ludvig Renbo Olsen, \email{r-pkgs@@ludvigolsen.dk}
 #' @export
 #' @family grouping functions
-#' @inheritParams group_factor
+#' @param data Data frame.
 #' @param p List or vector of partition sizes.
 #'  Given as whole number(s) and/or percentage(s) (\code{0} < \code{n} < \code{1}).
 #'  E.g. \eqn{c(0.2, 3, 0.1)}.
@@ -129,35 +129,44 @@ if(getRversion() >= "2.15.1")  utils::globalVariables(c("."))
 #'
 #' # Create data frame
 #' df <- data.frame(
-#'  "participant" = factor(rep(c('1','2', '3', '4', '5', '6'), 3)),
-#'  "age" = rep(sample(c(1:100), 6), 3),
-#'  "diagnosis" = rep(c('a', 'b', 'a', 'a', 'b', 'b'), 3),
-#'  "score" = sample(c(1:100), 3*6))
+#'   "participant" = factor(rep(c("1", "2", "3", "4", "5", "6"), 3)),
+#'   "age" = rep(sample(c(1:100), 6), 3),
+#'   "diagnosis" = factor(rep(c("a", "b", "a", "a", "b", "b"), 3)),
+#'   "score" = sample(c(1:100), 3 * 6)
+#' )
 #' df <- df %>% arrange(participant)
-#' df$session <- rep(c('1','2', '3'), 6)
+#' df$session <- rep(c("1", "2", "3"), 6)
 #'
 #' # Using partition()
 #'
 #' # Without balancing
-#' partitions <- partition(df, c(0.2,0.3))
+#' partitions <- partition(data = df, p = c(0.2, 0.3))
 #'
 #' # With cat_col
-#' partitions <- partition(df, c(0.5), cat_col = 'diagnosis')
+#' partitions <- partition(data = df, p = 0.5, cat_col = "diagnosis")
 #'
 #' # With id_col
-#' partitions <- partition(df, c(0.5), id_col = 'participant')
+#' partitions <- partition(data = df, p = 0.5, id_col = "participant")
 #'
 #' # With num_col
-#' partitions <- partition(df, c(0.5), num_col = 'score')
+#' partitions <- partition(data = df, p = 0.5, num_col = "score")
 #'
 #' # With cat_col and id_col
-#' partitions <- partition(df, c(0.5), cat_col = 'diagnosis',
-#'                         id_col = 'participant')
+#' partitions <- partition(
+#'   data = df,
+#'   p = 0.5,
+#'   cat_col = "diagnosis",
+#'   id_col = "participant"
+#' )
 #'
 #' # With cat_col, num_col and id_col
-#' partitions <- partition(df, c(0.5), cat_col = 'diagnosis',
-#'                         num_col = "score",
-#'                         id_col = 'participant')
+#' partitions <- partition(
+#'   data = df,
+#'   p = 0.5,
+#'   cat_col = "diagnosis",
+#'   num_col = "score",
+#'   id_col = "participant"
+#' )
 #'
 #' # Return data frame with grouping factor
 #' # with list_out = FALSE
@@ -166,29 +175,43 @@ if(getRversion() >= "2.15.1")  utils::globalVariables(c("."))
 #' # Check if additional extreme_pairing_levels
 #' # improve the numerical balance
 #' set.seed(2) # try with seed 1 as well
-#' partitions_1 <- partition(df, c(0.5), num_col = 'score',
-#'                           extreme_pairing_levels = 1,
-#'                           list_out = FALSE)
+#' partitions_1 <- partition(
+#'   data = df,
+#'   p = 0.5,
+#'   num_col = "score",
+#'   extreme_pairing_levels = 1,
+#'   list_out = FALSE
+#' )
 #' partitions_1 %>%
 #'   dplyr::group_by(.partitions) %>%
-#'   dplyr::summarise(sum_score = sum(score),
-#'                    mean_score = mean(score))
+#'   dplyr::summarise(
+#'     sum_score = sum(score),
+#'     mean_score = mean(score)
+#'   )
 #' set.seed(2) # try with seed 1 as well
-#' partitions_2 <- partition(df, c(0.5), num_col = 'score',
-#'                           extreme_pairing_levels = 2,
-#'                           list_out = FALSE)
+#' partitions_2 <- partition(
+#'   data = df,
+#'   p = 0.5,
+#'   num_col = "score",
+#'   extreme_pairing_levels = 2,
+#'   list_out = FALSE
+#' )
 #' partitions_2 %>%
 #'   dplyr::group_by(.partitions) %>%
-#'   dplyr::summarise(sum_score = sum(score),
-#'                    mean_score = mean(score))
-#'
+#'   dplyr::summarise(
+#'     sum_score = sum(score),
+#'     mean_score = mean(score)
+#'   )
 #' @importFrom dplyr group_by do %>%
-partition <- function(data, p=0.2, cat_col=NULL,
-                      num_col=NULL, id_col=NULL,
-                      id_aggregation_fn=sum,
-                      extreme_pairing_levels=1,
-                      force_equal=FALSE,
-                      list_out=TRUE) {
+partition <- function(data,
+                      p = 0.2,
+                      cat_col = NULL,
+                      num_col = NULL,
+                      id_col = NULL,
+                      id_aggregation_fn = sum,
+                      extreme_pairing_levels = 1,
+                      force_equal = FALSE,
+                      list_out = TRUE) {
 
   #
   # Balanced partitioning
@@ -201,33 +224,41 @@ partition <- function(data, p=0.2, cat_col=NULL,
   #        FALSE allows you to pass "p = 0.2" and get 2 partitions - 0.2 and 0.8
   #
 
-  if (!is.null(cat_col) && cat_col %ni% colnames(data)){
-    stop(paste0("cat_col: '", cat_col, "' is not in data"))}
-  if (!is.null(id_col) && id_col %ni% colnames(data)){
-    stop(paste0("id_col: '", id_col, "' is not in data"))}
-  if (!is.null(num_col) && num_col %ni% colnames(data)){
-    stop(paste0("num_col: '", num_col, "' is not in data"))}
-
+  check_partition(
+    data = data,
+    p = p,
+    cat_col = cat_col,
+    num_col = num_col,
+    id_col = id_col,
+    id_aggregation_fn = id_aggregation_fn,
+    extreme_pairing_levels = extreme_pairing_levels,
+    force_equal = force_equal,
+    list_out = list_out
+  )
 
   # If num_col is not NULL
-  if (!is.null(num_col)){
-
-    data <- create_num_col_groups(data, n=p, num_col=num_col, cat_col=cat_col,
-                                  id_col=id_col, col_name=".partitions",
-                                  id_aggregation_fn = id_aggregation_fn,
-                                  extreme_pairing_levels=extreme_pairing_levels,
-                                  method="l_sizes", unequal_method="last",
-                                  force_equal=force_equal,
-                                  pre_randomize = TRUE
-                                  )
-
+  if (!is.null(num_col)) {
+    data <- create_num_col_groups(
+      data = data,
+      n = p,
+      num_col = num_col,
+      cat_col = cat_col,
+      id_col = id_col,
+      col_name = ".partitions",
+      id_aggregation_fn = id_aggregation_fn,
+      extreme_pairing_levels = extreme_pairing_levels,
+      method = "l_sizes",
+      unequal_method = "last",
+      force_equal = force_equal,
+      pre_randomize = TRUE
+    )
   } else {
 
     # If cat_col is not NULL
-    if (!is.null(cat_col)){
+    if (!is.null(cat_col)) {
 
       # If id_col is not NULL
-      if (!is.null(id_col)){
+      if (!is.null(id_col)) {
 
         # Group by cat_col
         # For each group:
@@ -236,13 +267,18 @@ partition <- function(data, p=0.2, cat_col=NULL,
         # Group by new grouping factor '.partitions'
 
         data <- data %>%
-          group_by(!! as.name(cat_col)) %>%
-          do(group_uniques_(., n = p, id_col, method = 'l_sizes',
-                            col_name = '.partitions',
-                            force_equal = force_equal)) %>%
-          group_by(!! as.name('.partitions'))
+          group_by(!!as.name(cat_col)) %>%
+          do(group_uniques_(
+            data = .,
+            n = p,
+            id_col,
+            method = "l_sizes",
+            col_name = ".partitions",
+            force_equal = force_equal
+          )) %>%
+          group_by(!!as.name(".partitions"))
 
-      # If id_col is NULL
+        # If id_col is NULL
       } else {
 
         # Group by cat_col
@@ -250,73 +286,142 @@ partition <- function(data, p=0.2, cat_col=NULL,
         # .. and add grouping factor to data
 
         data <- data %>%
-          group_by(!! as.name(cat_col)) %>%
-          do(group(., n = p, method = 'l_sizes',
-                   randomize = TRUE,
-                   col_name = '.partitions',
-                   force_equal = force_equal))
+          group_by(!!as.name(cat_col)) %>%
+          do(group(
+            data = .,
+            n = p,
+            method = "l_sizes",
+            randomize = TRUE,
+            col_name = ".partitions",
+            force_equal = force_equal
+          ))
       }
 
-    # If cat_col is NULL
+      # If cat_col is NULL
     } else {
 
       # If id_col is not NULL
-      if (!is.null(id_col)){
+      if (!is.null(id_col)) {
 
         # Create groups of unique IDs
         # .. and add grouping factor to data
 
         data <- data %>%
-          group_uniques_(n = p, id_col, method = 'l_sizes',
-                         col_name = '.partitions',
-                         force_equal = force_equal)
+          group_uniques_(
+            n = p,
+            id_col = id_col,
+            method = "l_sizes",
+            col_name = ".partitions",
+            force_equal = force_equal
+          )
 
-      # If id_col is NULL
+        # If id_col is NULL
       } else {
 
 
         # Create groups from all the data points
         # .. and add grouping factor to data
 
-        data <- group(data, n = p,
-                      method = 'l_sizes',
-                      randomize = TRUE,
-                      col_name = '.partitions',
-                      force_equal = force_equal)
+        data <- group(
+          data = data,
+          n = p,
+          method = "l_sizes",
+          randomize = TRUE,
+          col_name = ".partitions",
+          force_equal = force_equal
+        )
       }
     }
   }
 
-  if (isTRUE(list_out)){
+  if (isTRUE(list_out)) {
 
     # If we have any NAs in .partitions
-    if (anyNA(data$.partitions)){
-
+    if (anyNA(data$.partitions)) {
       stop("NA in .partitions column.")
-
-    } else if (is.null(data[['.partitions']])){
-
+    } else if (is.null(data[[".partitions"]])) {
       stop("Column .partitions does not exist")
-
     } else {
-      plyr::llply(c(1:max(as.integer(data[['.partitions']]))), function(part){
+      return(
+        plyr::llply(c(1:max(as.integer(data[[".partitions"]]))), function(part) {
+          temp_data <- data[data$.partitions == part,]
 
-        temp_data <- data[data$.partitions == part,]
-
-        temp_data$.partitions <- NULL
-
-        temp_data %>% dplyr::ungroup() %>% return()
-
-      }) %>% return()
+          temp_data$.partitions <- NULL
 
+          temp_data %>%
+            dplyr::ungroup()
+      })
+      )
     }
-
-  } else {
-
-    # Return data
-    return(data)
-
   }
 
+  data
+}
+
+
+check_partition <- function(data,
+                            p,
+                            cat_col,
+                            num_col,
+                            id_col,
+                            id_aggregation_fn,
+                            extreme_pairing_levels,
+                            force_equal,
+                            list_out){
+
+  # Check arguments ####
+  assert_collection <- checkmate::makeAssertCollection()
+  checkmate::assert_data_frame(x = data, min.rows = 1, add = assert_collection)
+  checkmate::reportAssertions(assert_collection)
+  checkmate::assert_numeric(x = p, lower = 1/nrow(data), upper = nrow(data), any.missing = FALSE,
+                            finite = TRUE, add = assert_collection)
+  checkmate::assert_count(x = extreme_pairing_levels, positive = TRUE, add = assert_collection)
+  checkmate::assert_character(x = cat_col, min.len = 1, any.missing = FALSE,
+                              null.ok = TRUE, unique = TRUE,
+                              names = "unnamed", add = assert_collection)
+  checkmate::assert_string(x = num_col, na.ok = FALSE, min.chars = 1,
+                           null.ok = TRUE,  add = assert_collection)
+  checkmate::assert_string(x = id_col, na.ok = FALSE, min.chars = 1,
+                           null.ok = TRUE,  add = assert_collection)
+  checkmate::assert_function(x = id_aggregation_fn, add = assert_collection)
+  checkmate::assert_flag(x = force_equal, add = assert_collection)
+  checkmate::assert_flag(x = list_out, add = assert_collection)
+  checkmate::reportAssertions(assert_collection)
+  # TODO Could have a helper for adding this kind of message:
+  if (!is.null(cat_col) && length(setdiff(cat_col, colnames(data))) != 0){
+    assert_collection$push(paste0("'cat_col' column(s), '",
+                                  paste0(setdiff(cat_col, colnames(data)), collapse = ", "),
+                                  "', not found in 'data'."))
+  }
+  if (!is.null(num_col)){
+    if (num_col %ni% colnames(data)){
+      assert_collection$push(paste0("'num_col' column, '", num_col, "', not found in 'data'."))
+    }
+    checkmate::reportAssertions(assert_collection)
+    if (!checkmate::test_numeric(data[[num_col]])){
+      assert_collection$push(paste0("'num_col' column must be numeric."))
+    }
+  }
+  if (!is.null(id_col)) {
+    if (id_col %ni% colnames(data)){
+      assert_collection$push(paste0("'id_col' column, '", id_col, "', not found in 'data'."))
+    }
+    checkmate::reportAssertions(assert_collection)
+    checkmate::assert_factor(x = data[[id_col]], add = assert_collection)
+    if (!is.null(cat_col)) {
+      if (id_col %in% cat_col) {
+        assert_collection$push("'id_col' and 'cat_col' cannot contain the same column name.")
+      }
+      # Check that cat_col is constant within each ID
+      # Note: I tested and count() is faster than group_keys()
+      counts <- dplyr::count(data, !!as.name(id_col), !!as.name(cat_col))
+      if (nrow(counts) != length(unique(counts[[id_col]]))) {
+        assert_collection$push("The value in 'data[[cat_col]]' must be constant within each ID.")
+      }
+    }
+  }
+  checkmate::reportAssertions(assert_collection)
+  # End of argument checks ####
+
 }
 
