@@ -495,19 +495,21 @@ test_that("repeated folding works in fold()", {
 test_that("bootstrap test of num_col works", {
 
   # Takes 4 seconds, so we disable it for now.
-  testthat::skip(message = "Skipping bootstrapped numerical balancing test in fold()")
+  # testthat::skip(message = "Skipping bootstrapped numerical balancing test in fold()")
 
+  xpectr::set_test_seed(1)
   df <- data.frame(
     "participant" = factor(rep(1:100, 100)),
     "diagnosis" = factor(rep(c("a", "b", "c", "d", "e"), 2000)),
     "age" = rep(sample(100), 100)
   )
 
-  # Single
+  # Single ####
+
   xpectr::set_test_seed(1)
   df_folded <- fold(df, 3, num_col = "age")
 
-  for (i in 1:10) {
+  mean_ages <- plyr::ldply(1:10, function(i){
     xpectr::set_test_seed(i)
     df_folded <- fold(df, 0.5,
       cat_col = "diagnosis", num_col = "age",
@@ -521,14 +523,58 @@ test_that("bootstrap test of num_col works", {
         sd_age = sd(age)
       )
 
-    expect_true(is_between_(age_distribution$mean_age[1], 50, 52))
-    expect_true(is_between_(age_distribution$mean_age[2], 49, 51))
-  }
+    age_distribution$mean_age
+  })
 
-  for (i in 1:10) {
+
+  ## Testing 'mean_ages'                                                    ####
+  ## Initially generated by xpectr
+  xpectr::set_test_seed(42)
+  # Testing class
+  expect_equal(
+    class(mean_ages),
+    "data.frame",
+    fixed = TRUE)
+  # Testing column values
+  expect_equal(
+    mean_ages[["V1"]],
+    c(50.6, 50.42, 50.52, 50.62, 50.6, 50.54, 50.56, 50.68, 50.2, 50.48),
+    tolerance = 1e-4)
+  expect_equal(
+    mean_ages[["V2"]],
+    c(50.4, 50.58, 50.48, 50.38, 50.4, 50.46, 50.44, 50.32, 50.8, 50.52),
+    tolerance = 1e-4)
+  # Testing column names
+  expect_equal(
+    names(mean_ages),
+    c("V1", "V2"),
+    fixed = TRUE)
+  # Testing column classes
+  expect_equal(
+    xpectr::element_classes(mean_ages),
+    c("numeric", "numeric"),
+    fixed = TRUE)
+  # Testing column types
+  expect_equal(
+    xpectr::element_types(mean_ages),
+    c("double", "double"),
+    fixed = TRUE)
+  # Testing dimensions
+  expect_equal(
+    dim(mean_ages),
+    c(10L, 2L))
+  # Testing group keys
+  expect_equal(
+    colnames(dplyr::group_keys(mean_ages)),
+    character(0),
+    fixed = TRUE)
+  ## Finished testing 'mean_ages'                                           ####
+
+  mean_ages <- plyr::ldply(1:10, function(i){
     xpectr::set_test_seed(i)
     df_folded <- fold(df, 5,
-      cat_col = "diagnosis", num_col = "age",
+      cat_col = "diagnosis",
+      num_col = "age",
       id_col = "participant"
     )
 
@@ -539,20 +585,75 @@ test_that("bootstrap test of num_col works", {
         sd_age = sd(age)
       )
 
-    expect_true(is_between_(age_distribution$mean_age[1], 47.5, 53.5))
-    expect_true(is_between_(age_distribution$mean_age[2], 47.5, 53.5))
-    expect_true(is_between_(age_distribution$mean_age[3], 47.5, 53.5))
-    expect_true(is_between_(age_distribution$mean_age[4], 47.5, 53.5))
-    expect_true(is_between_(age_distribution$mean_age[5], 47.5, 53.5))
-  }
+    age_distribution$mean_age
+  })
 
-  # With two levels of extreme pairing
 
-  for (i in 1:10) {
+  ## Testing 'mean_ages'                                                    ####
+  ## Initially generated by xpectr
+  xpectr::set_test_seed(42)
+  # Testing class
+  expect_equal(
+    class(mean_ages),
+    "data.frame",
+    fixed = TRUE)
+  # Testing column values
+  expect_equal(
+    mean_ages[["V1"]],
+    c(49.4, 51.2, 50.9, 51.5, 50, 49.2, 50.15, 50.2, 49.5, 51.8),
+    tolerance = 1e-4)
+  expect_equal(
+    mean_ages[["V2"]],
+    c(49.7, 49.5, 50.45, 51.4, 49.75, 51.15, 51.05, 49.9, 49, 49.15),
+    tolerance = 1e-4)
+  expect_equal(
+    mean_ages[["V3"]],
+    c(52.6, 50.4, 49.7, 50, 51, 50.05, 50.35, 52.65, 51.45, 50.05),
+    tolerance = 1e-4)
+  expect_equal(
+    mean_ages[["V4"]],
+    c(50.55, 50.35, 50.9, 50.05, 50.55, 50.8, 50.75, 50.45, 50, 50.85),
+    tolerance = 1e-4)
+  expect_equal(
+    mean_ages[["V5"]],
+    c(50.25, 51.05, 50.55, 49.55, 51.2, 51.3, 50.2, 49.3, 52.55, 50.65),
+    tolerance = 1e-4)
+  # Testing column names
+  expect_equal(
+    names(mean_ages),
+    c("V1", "V2", "V3", "V4", "V5"),
+    fixed = TRUE)
+  # Testing column classes
+  expect_equal(
+    xpectr::element_classes(mean_ages),
+    c("numeric", "numeric", "numeric", "numeric", "numeric"),
+    fixed = TRUE)
+  # Testing column types
+  expect_equal(
+    xpectr::element_types(mean_ages),
+    c("double", "double", "double", "double", "double"),
+    fixed = TRUE)
+  # Testing dimensions
+  expect_equal(
+    dim(mean_ages),
+    c(10L, 5L))
+  # Testing group keys
+  expect_equal(
+    colnames(dplyr::group_keys(mean_ages)),
+    character(0),
+    fixed = TRUE)
+  ## Finished testing 'mean_ages'                                           ####
+
+
+  # With two levels of extreme pairing ####
+
+  mean_ages <- plyr::ldply(1:10, function(i){
     xpectr::set_test_seed(i)
     df_folded <- fold(df, 5,
-      cat_col = "diagnosis", num_col = "age",
-      id_col = "participant", extreme_pairing_levels = 2
+      cat_col = "diagnosis",
+      num_col = "age",
+      id_col = "participant",
+      extreme_pairing_levels = 2
     )
 
     age_distribution <- df_folded %>%
@@ -562,16 +663,69 @@ test_that("bootstrap test of num_col works", {
         sd_age = sd(age)
       )
 
-    expect_true(is_between_(age_distribution$mean_age[1], 49, 51.5))
-    expect_true(is_between_(age_distribution$mean_age[2], 49, 51.5))
-    expect_true(is_between_(age_distribution$mean_age[3], 49, 51.5))
-    expect_true(is_between_(age_distribution$mean_age[4], 49, 51.5))
-    expect_true(is_between_(age_distribution$mean_age[5], 49, 51.5))
-  }
+    age_distribution$mean_age
+  })
+
+
+  ## Testing 'mean_ages'                                                    ####
+  ## Initially generated by xpectr
+  xpectr::set_test_seed(42)
+  # Testing class
+  expect_equal(
+    class(mean_ages),
+    "data.frame",
+    fixed = TRUE)
+  # Testing column values
+  expect_equal(
+    mean_ages[["V1"]],
+    c(50.5, 51.15, 50.05, 50.15, 50.9, 50, 50.05, 50.6, 50.5, 50.4),
+    tolerance = 1e-4)
+  expect_equal(
+    mean_ages[["V2"]],
+    c(50.4, 50.2, 50.65, 50.2, 50.05, 49.9, 50.4, 50.9, 50.25, 50.8),
+    tolerance = 1e-4)
+  expect_equal(
+    mean_ages[["V3"]],
+    c(50.4, 50.35, 50.35, 50.45, 50.3, 51.3, 50.3, 50.6, 50.25, 50.05),
+    tolerance = 1e-4)
+  expect_equal(
+    mean_ages[["V4"]],
+    c(50.55, 50.35, 50.95, 50.8, 50.9, 50.5, 50.4, 50.05, 50.95, 50.35),
+    tolerance = 1e-4)
+  expect_equal(
+    mean_ages[["V5"]],
+    c(50.65, 50.45, 50.5, 50.9, 50.35, 50.8, 51.35, 50.35, 50.55, 50.9),
+    tolerance = 1e-4)
+  # Testing column names
+  expect_equal(
+    names(mean_ages),
+    c("V1", "V2", "V3", "V4", "V5"),
+    fixed = TRUE)
+  # Testing column classes
+  expect_equal(
+    xpectr::element_classes(mean_ages),
+    c("numeric", "numeric", "numeric", "numeric", "numeric"),
+    fixed = TRUE)
+  # Testing column types
+  expect_equal(
+    xpectr::element_types(mean_ages),
+    c("double", "double", "double", "double", "double"),
+    fixed = TRUE)
+  # Testing dimensions
+  expect_equal(
+    dim(mean_ages),
+    c(10L, 5L))
+  # Testing group keys
+  expect_equal(
+    colnames(dplyr::group_keys(mean_ages)),
+    character(0),
+    fixed = TRUE)
+  ## Finished testing 'mean_ages'                                           ####
+
 
   # With three levels of extreme pairing
 
-  for (i in 1:10) {
+  mean_ages <- plyr::ldply(1:10, function(i){
     xpectr::set_test_seed(i)
     expect_error(
       fold(df, 5,
@@ -592,17 +746,76 @@ test_that("bootstrap test of num_col works", {
         sd_age = sd(age)
       )
 
-    expect_true(is_between_(age_distribution$mean_age[1], 49, 51.5))
-    expect_true(is_between_(age_distribution$mean_age[2], 49, 51.5))
-    expect_true(is_between_(age_distribution$mean_age[3], 49, 51.5))
-    expect_true(is_between_(age_distribution$mean_age[4], 49, 51.5))
-    expect_true(is_between_(age_distribution$mean_age[5], 49, 51.5))
-  }
+    age_distribution$mean_age
+  })
 
-  for (i in 1:10) {
+
+  ## Testing 'mean_ages'                                                    ####
+  ## Initially generated by xpectr
+  xpectr::set_test_seed(42)
+  # Testing class
+  expect_equal(
+    class(mean_ages),
+    "data.frame",
+    fixed = TRUE)
+  # Testing column values
+  expect_equal(
+    mean_ages[["V1"]],
+    c(50.5175, 50.4745, 50.502, 50.504, 50.506, 50.5095, 50.5285, 50.485,
+      50.5025, 50.5035),
+    tolerance = 1e-4)
+  expect_equal(
+    mean_ages[["V2"]],
+    c(50.506, 50.515, 50.488, 50.5035, 50.527, 50.547, 50.4985, 50.51,
+      50.49, 50.5215),
+    tolerance = 1e-4)
+  expect_equal(
+    mean_ages[["V3"]],
+    c(50.4975, 50.464, 50.508, 50.499, 50.5045, 50.4515, 50.504, 50.4835,
+      50.5165, 50.498),
+    tolerance = 1e-4)
+  expect_equal(
+    mean_ages[["V4"]],
+    c(50.497, 50.5315, 50.5055, 50.4885, 50.481, 50.5035, 50.4895, 50.5025,
+      50.521, 50.4775),
+    tolerance = 1e-4)
+  expect_equal(
+    mean_ages[["V5"]],
+    c(50.482, 50.515, 50.4965, 50.505, 50.4815, 50.4885, 50.4795, 50.519,
+      50.47, 50.4995),
+    tolerance = 1e-4)
+  # Testing column names
+  expect_equal(
+    names(mean_ages),
+    c("V1", "V2", "V3", "V4", "V5"),
+    fixed = TRUE)
+  # Testing column classes
+  expect_equal(
+    xpectr::element_classes(mean_ages),
+    c("numeric", "numeric", "numeric", "numeric", "numeric"),
+    fixed = TRUE)
+  # Testing column types
+  expect_equal(
+    xpectr::element_types(mean_ages),
+    c("double", "double", "double", "double", "double"),
+    fixed = TRUE)
+  # Testing dimensions
+  expect_equal(
+    dim(mean_ages),
+    c(10L, 5L))
+  # Testing group keys
+  expect_equal(
+    colnames(dplyr::group_keys(mean_ages)),
+    character(0),
+    fixed = TRUE)
+  ## Finished testing 'mean_ages'                                           ####
+
+  age_distributions <- plyr::ldply(1:4, function(i){
     xpectr::set_test_seed(i)
-    df_folded <- fold(df, i,
-      cat_col = "diagnosis", num_col = "age", extreme_pairing_levels = 1
+    df_folded <- fold(df, k = i,
+      cat_col = "diagnosis",
+      num_col = "age",
+      extreme_pairing_levels = 1
     )
 
     age_distribution <- df_folded %>%
@@ -612,8 +825,7 @@ test_that("bootstrap test of num_col works", {
         sd_age = sd(age)
       )
 
-    expect_true(is_between_(min(age_distribution$mean_age), 49, 51.5))
-    expect_true(is_between_(max(age_distribution$mean_age), 49, 51.5))
+    current_rows <- list(age_distribution)
 
     xpectr::set_test_seed(i)
     df_folded <- fold(df, i,
@@ -627,8 +839,7 @@ test_that("bootstrap test of num_col works", {
         sd_age = sd(age)
       )
 
-    expect_true(is_between_(min(age_distribution$mean_age), 49, 51.5))
-    expect_true(is_between_(max(age_distribution$mean_age), 49, 51.5))
+    current_rows <- c(current_rows, list(age_distribution))
 
     xpectr::set_test_seed(i)
     df_folded <- fold(df, i,
@@ -642,9 +853,66 @@ test_that("bootstrap test of num_col works", {
         sd_age = sd(age)
       )
 
-    expect_true(is_between_(min(age_distribution$mean_age), 49, 51.5))
-    expect_true(is_between_(max(age_distribution$mean_age), 49, 51.5))
-  }
+    current_rows <- dplyr::bind_rows(c(current_rows, list(age_distribution)))
+    current_rows
+  })
+
+
+
+  ## Testing 'age_distributions'                                            ####
+  ## Initially generated by xpectr
+  xpectr::set_test_seed(42)
+  # Testing class
+  expect_equal(
+    class(age_distributions),
+    "data.frame",
+    fixed = TRUE)
+  # Testing column values
+  expect_equal(
+    age_distributions[[".folds"]],
+    structure(c(1L, 1L, 1L, 1L, 2L, 1L, 2L, 1L, 2L, 1L, 2L, 3L, 1L,
+      2L, 3L, 1L, 2L, 3L, 1L, 2L, 3L, 4L, 1L, 2L, 3L, 4L, 1L, 2L,
+      3L, 4L), .Label = c("1", "2", "3", "4"), class = "factor"))
+  expect_equal(
+    age_distributions[["mean_age"]],
+    c(50.5, 50.5, 50.5, 50.488, 50.512, 50.5016, 50.4984, 50.5078, 50.4922,
+      50.40912, 50.52024, 50.5707, 50.49295, 50.4997, 50.50736, 50.52116,
+      50.5093, 50.46957, 50.5368, 50.5944, 50.3496, 50.5192, 50.4924,
+      50.5444, 50.472, 50.4912, 50.504, 50.5056, 50.5152, 50.4752),
+    tolerance = 1e-4)
+  expect_equal(
+    age_distributions[["sd_age"]],
+    c(28.86751, 28.86751, 28.86751, 28.78545, 28.95221, 28.8557, 28.88221,
+      28.89085, 28.84704, 28.95847, 29.08568, 28.56392, 28.9281, 28.63966,
+      29.04209, 28.76932, 28.75625, 29.08426, 28.89381, 28.46876,
+      28.95188, 29.16791, 28.59038, 29.28341, 28.66115, 28.94721,
+      28.93272, 28.86576, 28.63692, 29.05039),
+    tolerance = 1e-4)
+  # Testing column names
+  expect_equal(
+    names(age_distributions),
+    c(".folds", "mean_age", "sd_age"),
+    fixed = TRUE)
+  # Testing column classes
+  expect_equal(
+    xpectr::element_classes(age_distributions),
+    c("factor", "numeric", "numeric"),
+    fixed = TRUE)
+  # Testing column types
+  expect_equal(
+    xpectr::element_types(age_distributions),
+    c("integer", "double", "double"),
+    fixed = TRUE)
+  # Testing dimensions
+  expect_equal(
+    dim(age_distributions),
+    c(30L, 3L))
+  # Testing group keys
+  expect_equal(
+    colnames(dplyr::group_keys(age_distributions)),
+    character(0),
+    fixed = TRUE)
+  ## Finished testing 'age_distributions'                                   ####
 
 
 
